﻿define("aiassistant/editors/AiTextarea", [
    "dojo/_base/declare", // declare
    "dojo/dom-style", // domStyle.set
    "dojo/on",
    'dijit/_TemplatedMixin',
    "dijit/form/_ExpandingTextAreaMixin",
    "dijit/form/SimpleTextarea",
    "dijit/DropDownMenu",
    "dijit/MenuItem",
    "dijit/form/DropDownButton",
    "dojo/text!./templates/AiTextarea.html",
    "epi/shell/XhrWrapper",
    "dojo/topic",
    "dojo/dom",
    "epi/dependency",
    "dojo/aspect",
    "xstyle/css!./templates/AiText.css"
], function (declare, domStyle, on, _TemplatedMixin, _ExpandingTextAreaMixin, SimpleTextarea, DropDownMenu, MenuItem, DropDownButton, template, XhrWrapper, topic, dom, dependency, aspect) {

    // module:
    //		dijit/form/Textarea

    var xhr = new XhrWrapper();

    return declare("aiassistant/editors/AiTextarea", [SimpleTextarea, _ExpandingTextAreaMixin, _TemplatedMixin], {
        // summary:
        //		A textarea widget that adjusts it's height according to the amount of data.
        //
        // description:
        //		A textarea that dynamically expands/contracts (changing it's height) as
        //		the user types, to display all the text without requiring a scroll bar.
        //
        //		Takes nearly all the parameters (name, value, etc.) that a vanilla textarea takes.
        //		Rows is not supported since this widget adjusts the height.
        //
        // example:
        // |	<textarea data-dojo-type="dijit/form/TextArea">...</textarea>
        oldValue: "",
        isLoading: false,
        // TODO: for 2.0, rename this to ExpandingTextArea, and rename SimpleTextarea to TextArea

        baseClass: "dijitTextBox dijitTextArea dijitExpandingTextArea AiClass",
        aiJsonResult: null,

        // Override SimpleTextArea.cols to default to width:100%, for backward compatibility
        cols: "",
        // Override
        templateString: template,
        //_contextChangedHandler: null,
        postCreate: function () {
            oldValue = this.get("value");
            this.aiStatus.innerHTML = "<- click the button to get AI Assistance";

            var configArr = this.shortcuts.split("||")
            if (configArr[0] !== "") { //do not show dropdown if no content in drop
                var self = this;
                var menu = new DropDownMenu({ class: "aiDD" });

                // Extend the MenuItem to include a value property
                var ValueMenuItem = declare(MenuItem, {
                    _setValueAttr: function (value) {
                        this._set("value", value);
                    }
                });

                configArr.forEach(function (label) {
                    var menuItem = new ValueMenuItem({
                        label: label,
                        value: label,
                        onClick: function () {
                            self._onAiSuggestButtonClick(null, this.value);
                        }
                    });

                    menu.addChild(menuItem);
                });

                var dropdownButton = new DropDownButton({
                    label: "",
                    showLabel: false,
                    dropDown: menu
                }, this.dropdownMenuButton);

                dropdownButton.startup();
                dropdownButton.clickOnDrop = 0;

                //this bellow is a work-around that the textbox get focus if you press the dropdown first, making the dropdown close.
                on(dropdownButton, "focus", function () { //this will be triggered before the textbox focus
                    dropdownButton.clickOnDrop++; //so count up
                });

                on(this, "blur", function () {//leaving the prop, reset
                    dropdownButton.closeDropDown(false);
                    dropdownButton.clickOnDrop = 0;
                });

                on(this.textbox, "focus", function () {
                    if (dropdownButton.clickOnDrop > 0) {//will only be 1 if direct hit on dropdown on focus
                        dropdownButton.closeDropDown(false);
                        dropdownButton.openDropDown();
                    }
                    dropdownButton.clickOnDrop = -100;
                });
            }

            this.inherited(arguments);
        },
        _onTextBoxBlur: function () {

        },
        _onAiSuggestButtonClick: function (event, /*prompt from shortcuts*/ shortcut) {

            var text = this.get("value").trim();
            this._doCall(text, shortcut);

            this.setAiStatus("Let me work on it");
            setTimeout(this.setAiStatus, 1000, "Let me work on it.", this, true);
            setTimeout(this.setAiStatus, 1400, "Let me work on it..", this, true);
            setTimeout(this.setAiStatus, 1800, "Let me work on it...", this, true);
            setTimeout(this.setAiStatus, 3000, "Let me work on it... I need just a few more seconds...", this, true);
            return false;
        },
        _onAiUseSuggestionButtonClick: function () {
            this.setAiStatus(this.aiJsonResult.useStatus);
            this.oldValue = this.aiSuggestion.value;
            this._setValueAttr(this.oldValue, true);
            this.hideResultBlock();
            this._sendStats(this.aiJsonResult);
            return false;

        },
        _handleOnChange: function (newValue, priorityChange) {

            if (priorityChange && newValue != this.value && newValue != this.oldValue && !this.isLoading) {
                this.setAiStatus("Processing ... ");
                this._doCall(newValue);
            }

            this.inherited(arguments);

        },
        _doCall: function (newValue, shortcut) {
            var self = this;
            this.isLoading = true;
            this.setAiSpinner(true);
            //this.hideResultBlock();
            xhr.xhrPost({
                url: '/episerver/epicweb.optimizely.aiassistant/suggest',
                handles: 'json',
                postData: dojo.toJson({ correlationid: this.correlationid, text: newValue, lang: dependency.resolve("epi.cms.contentEditing.command.Editing").model.languageContext.language, contentid: dependency.resolve("epi.cms.contentEditing.command.Editing").model.contentLink, propname: this.get("name"), shortcut: shortcut }),
                headers: { "Content-Type": "application/json", "Accept": "application/json" },
                load: function (jsonText) {
                    self.isLoading = false;//we got result
                    self.aiJsonResult = JSON.parse(jsonText);
                    self.setAiStatus(self.aiJsonResult.aiStatus);//get this from result
                    self.setUseButton(self.aiJsonResult.useButton);
                    self.setAiLicenseHtml(self.aiJsonResult.licenseHtml);
                    self.setAiLimitationHtml(self.aiJsonResult.limitations);
                    if (self.aiJsonResult.openResult) {
                        self.setAiSuggestion(self.aiJsonResult.text);
                        self.showResultBlock();
                    }
                    else {
                        self.hideResultBlock();
                    }
                    self.setAiSpinner(false);
                }
            });
        },
        _sendStats: function (json) {
            xhr.xhrPost({
                url: '/episerver/epicweb.optimizely.aiassistant/stats',
                handles: 'json',
                postData: dojo.toJson(json),
                headers: { "Content-Type": "application/json", "Accept": "application/json" },
                load: function (jsonText) {
                   // JSON.parse(jsonText);
                }
            });
        },
        setAiStatus(value, plugin, whileloading) {

            if (whileloading) {//only if loading
                if (plugin.isLoading)//and is still loading
                    plugin.aiStatus.innerHTML = value;//then show this
            }
            else
                this.aiStatus.innerHTML = value;
        },
        setAiSpinner(spin) {
            dojo.toggleClass(this.aiSpinner, "spin", spin);
        },
        setAiSuggestion(value) {
            this.aiSuggestion.innerHTML = value;
        },
        setUseButton(value) {
            this.aibutton.innerHTML = value;
        },
        setAiLicenseHtml(value) {
            if (value == "" || value == null || value == undefined) {
                this.aiLicense.style.display = "none";
            } else {
                this.aiLicense.style.display = "";
                this.aiLicense.innerHTML = value;
            }
        },
        setAiLimitationHtml(value) {
            if (value == "" || value == null || value == undefined) {
                this.aiLimitation.style.display = "none";
            } else {
                this.aiLimitation.style.display = "";
                this.aiLimitation.innerHTML = value;
            }
        },
        hideResultBlock() {
            this.aiDivResult.style.display = "none";
        },

        showResultBlock() {
            this.aiDivResult.style.display = "block";
        },
        buildRendering: function () {
            this.inherited(arguments);

            this.baseClass = this.baseClass + " type" + this.inputtype

            if (this.inputtype == "input") { //act as inputfield, no line breaks
                this.textbox.addEventListener("keyup", function (event) {
                    const keyCode = event.keyCode || event.which;

                    // Remove line breaks on Enter key (keyCode 13)
                    if (keyCode === 13) {
                        event.preventDefault();
                        this.value = this.value.replace(/(\r\n|\n|\r)/gm, "");
                    }
                })
                domStyle.set(this.textbox, { whiteSpace: 'nowrap', overflowY: 'hidden', overflowX: 'auto', boxSizing: 'border-box', MsBoxSizing: 'border-box', WebkitBoxSizing: 'border-box', MozBoxSizing: 'border-box' });
            }
            else
            // tweak textarea style to reduce browser differences
            domStyle.set(this.textbox, { overflowY: 'hidden', overflowX: 'auto', boxSizing: 'border-box', MsBoxSizing: 'border-box', WebkitBoxSizing: 'border-box', MozBoxSizing: 'border-box' });
        }
    });

});