﻿"use strict";

var _searchcounter = 0;

var initpagescript = async function () {


    const iframe = document.getElementById('mainframe');

    const selectUrl = getQueryStringParameterByName('selectUrl');
    const version = getQueryStringParameterByName('version');
    const defaultText = getQueryStringParameterByName('defaultText');
    const propname = getQueryStringParameterByName('propname');
    const contentid = getQueryStringParameterByName('contentid');
    const lang = getQueryStringParameterByName('lang');
    const correlationid = getQueryStringParameterByName("correlationid");

    //make it global
    window.pageData = {
        "correlationid": correlationid,
        "propname": propname,
        "lang": lang,
        "contentid": contentid
    };


    // init messaging
    window.addEventListener('message', messageEvent);

    /*
     * Handle message events.
     *
     * Forwards from opener to Iframe or returns from Iframe to opener.
     */
    function messageEvent(message) {
        if (verifyInputMessage(message)) {
            // forward message -> to Iframe
            if (iframe)
                iframe.contentWindow.postMessage(message.data, ivOrigin);
        } else {
            // reply to opener <- from Iframe
            if (window.opener)
                window.opener.postMessage(message.data, location.origin);
            else
                window.parent.postMessage(message.data, location.origin);
        }
    }


    /*
     * Verifies a message as input to Iframe:
     * Check that message origin is from us and make sure the message data is a valid input command.
     */
    function verifyInputMessage(message) {
        return message.origin === location.origin;
    }

    /*
     * Helper method to get specific query parameter from querystring
     */
    function getQueryStringParameterByName(name) {
        const url = window.location.href;
        name = name.replace(/[\[\]]/g, '\\$&');
        const regex = new RegExp('[?&]' + name + '(=([^&#]*)|&|#|$)');
        const results = regex.exec(url);

        if (!results) {
            return '';
        }

        if (!results[2]) {
            return '';
        }

        return decodeURIComponent(results[2].replace(/\+/g, ' '));
    }

    function initFunction() {
        const url = "/episerver/epicweb.optimizely.aiassistant/imagegeneration/init?correlationid=" + correlationid;

        hideImageLists();
        showOverlay();

        fetch(url, {
            method: "GET",
            headers: {
                "Content-Type": "application/json"
            }
        })
            .then((response) => {

                hideOverlay();

                response.json().then(data => {
                    var div = document.getElementById('aiImageResponse');
                    if (!isNullOrEmpty(data.limitations)) {
                        var pElement = document.createElement('p');
                        pElement.innerHTML = data.limitations;
                        div.appendChild(pElement);
                    }
                    if (data.enableGenerate) {
                        document.getElementById('btnGenerate').disabled = data.EnableGenerate;
                        document.getElementById('imgAiImageForm_txtSearch').disabled = data.EnableGenerate;
                        showImageLists();
                        if (defaultText !== "") {
                            document.getElementById('imgAiImageForm_txtSearch').value = defaultText;
                            _suggestImages();
                        }
                    }
                    if (!isNullOrEmpty(data.licenseHtml)) {
                        var pElement = document.createElement('p');
                        pElement.className = "red";
                        pElement.innerHTML = data.licenseHtml;
                        div.appendChild(pElement);
                    }
                });

            })
            .catch((error) => {
                console.error("Error:", error);
            });
    }


    function _suggestImages() {
        var searchEl = document.getElementById('imgAiImageForm_txtSearch');
        const url = "/episerver/epicweb.optimizely.aiassistant/imagegeneration/suggest";
        const data = {
            "text": searchEl.value,
            "correlationid": correlationid,
            "propname": propname,
            "lang": lang,
            "contentid": contentid
        };

        showOverlay();

        fetch(url, {
            method: "POST",
            headers: {
                "Content-Type": "application/json"
            },
            body: JSON.stringify(data)
        })
            .then((response) => {

                _unselectAsset();
                hideImageLists();
                hideOverlay();

                if (!response.ok) {
                    //debugger
                    response.json().then(data => {
                        document.getElementById('errormessage').innerHTML = data.error.message;
                        throw new Error(data.error.message);
                    });
                    return;
                }

                if (defaultText !== "" && selectUrl !== "" && _searchcounter == 0) {//this we have probably opened from TinyMce and had an image selected
                    showImageLists();
                }

                _searchcounter++;

                var div = document.getElementById('aiImageResponse');
                const ulElement = document.createElement('ul');
                ulElement.className = "imagelist";
                var txtInput = document.getElementById('imgAiImageForm_txtSearch');
                response.json().then(data => {
                    for (var i = 0; i <= data.images.length - 1; i++) {
                        const liElement = document.createElement('li');
                        const imgEl = document.createElement('img');
                        imgEl.src = data.images[i];
                        imgEl.className = "selectable";
                        imgEl.dataset.name = txtInput.value;
                        liElement.appendChild(imgEl);
                        ulElement.appendChild(liElement);
                    }
                    div.appendChild(ulElement);
                });

            })
            .catch((error) => {
                console.error("Error:", error);
                document.getElementById('errormessage').innerHTML = error;
            });
        return false;
    }


    try {

        window._suggestImages = _suggestImages;

        if (!isNullOrEmpty(selectUrl)) {

            var div = document.getElementById('aiImageResponse');
            const ulElement = document.createElement('ul');
            ulElement.className = "imagelist";
            const liElement = document.createElement('li');
            const imgEl = document.createElement('img');
            imgEl.src = selectUrl;
            imgEl.className = "selectable noUse";
            liElement.appendChild(imgEl);
            ulElement.appendChild(liElement);
            div.appendChild(ulElement);
        }

        initFunction();

        //here do init stuff
        console.log("AI Assistant version =>", version);

        // Attach the event to the body element
        document.body.addEventListener('click', function (event) {
            // Check if the clicked element is an image
            //console.log('body clicked!', event.target.tagName, event.target.className);

            if (event.target.tagName === 'IMG' && event.target.classList.contains("selectable")) {

                if (event.target.classList.contains('selected')) {
                    _unselectAsset();
                }
                else {
                    _unselectAsset();
                    event.target.classList.toggle('selected');
                    if (!event.target.classList.contains("noUse"))//do not use this one
                    {
                        _selectAsset(event.target.src, event.target.dataset.name);
                    }
                    //add buttons for variations
                    var templateElement = document.getElementById('buttonTemplate');
                    var container = document.createElement('div');
                    container.className = "actions";
                    container.innerHTML = templateElement.innerHTML;
                    event.target.parentNode.appendChild(container);
                }     
            }
        });



    } catch (error) {

        // Log error
        console.error(error);
        document.getElementById('errormessage').innerHTML = error;
    }
}

function _selectAsset(path, name) {
    var data = { "action": "selected", "path": path, "name": name };
    if (window.opener)
        window.opener.postMessage(data, location.origin);
    else
        window.parent.postMessage(data, location.origin);
}

function _unselectAsset() {

    var firstSelectedImg = document.querySelector('img.selected');
    // Check if the element exists
    if (firstSelectedImg) {
        // Perform actions with the first selected image
        //console.log("Unselected", firstSelectedImg.src);
        firstSelectedImg.classList.toggle('selected');
    } 

    // Find elements using a CSS selector
    var elementsToRemove = document.querySelectorAll('.actions');

    // Remove each element from its parent
    elementsToRemove.forEach(function (element) {
        element.parentNode.removeChild(element);
    });

    var data = { "action": "unselected", "path": "" };
    if (window.opener)
        window.opener.postMessage(data, location.origin);
    else
        window.parent.postMessage(data, location.origin);
}

function hideImageLists() {

    document.getElementById('errormessage').innerHTML = "";
    // Get all ul elements with the class "imagelist"
    var ulList = document.querySelectorAll('ul.imagelist');

    // Iterate through each ul element
    ulList.forEach(function (ulElement) {
        // Toggle the class "hidden" on the ul element
        if (!ulElement.classList.contains('hidden'))
            ulElement.classList.toggle('hidden');
    });
    if (_searchcounter > 0) {
        var btnHistory = document.getElementById('btnHistory');
        if (btnHistory)
            btnHistory.classList.remove("hidden");
    }
}

function showImageLists() {
    // Get all ul elements with the class "imagelist"
    var ulList = document.querySelectorAll('ul.imagelist');

    // Iterate through each ul element
    ulList.forEach(function (ulElement) {
        // Toggle the class "hidden" on the ul element
        if (ulElement.classList.contains('hidden')) {
            ulElement.classList.toggle('hidden');
        }
    });
}

function _enlarge(el) {
    showImageOverlay(el.parentNode.parentNode.firstElementChild.src);
}

function showImageOverlay(imageUrl) {
    var overlay = document.getElementById('overlay2');
    var overlayImage = document.getElementById('overlayImage');
    overlayImage.src = imageUrl;
    overlayImage.addEventListener('click', hideImageOverlay);
    overlay.addEventListener('click', hideImageOverlay);
    overlay.style.display = 'flex';
}

function hideImageOverlay() {
    var overlay = document.getElementById('overlay2');
    overlay.style.display = 'none';
}

function _generateVariation(el) {
    _onImageVariation(el.parentNode.parentNode.firstElementChild.src);
}

function _onImageVariation(imagePath) {
    const url = "/episerver/epicweb.optimizely.aiassistant/imagegeneration/variations";

    const data = {
        "text": imagePath,
        "correlationid": window.pageData.correlationid,
        "propname": window.pageData.propname,
        "lang": window.pageData.lang,
        "contentid": window.pageData.contentid
    };

    showOverlay();
    fetch(url, {
            method: "POST",
            headers: {
                "Content-Type": "application/json"
            },
            body: JSON.stringify(data)
        }).then((response) => {


            _unselectAsset();
            hideImageLists();
            hideOverlay();

            if (!response.ok) {
                //debugger
                response.json().then(data => {
                    let msg = data.message || data.error.message;
                    if (msg.indexOf('localhost') > 0)
                        msg += " - Upload of images does not work from localhost.";
                    document.getElementById('errormessage').innerHTML = msg;
                    throw new Error(msg);
                });
                return;
            }

            _searchcounter++;

            response.json().then(data => {
                var div = document.getElementById('aiImageResponse');
                const ulElement = document.createElement('ul');
                ulElement.className = "imagelist";
                var txtInput = document.getElementById('imgAiImageForm_txtSearch');
                for (var i = 0; i <= data.images.length - 1; i++) {
                    const liElement = document.createElement('li');
                    const imgEl = document.createElement('img');
                    imgEl.className = "selectable";
                    imgEl.dataset.name = txtInput.value;
                    imgEl.src = data.images[i];
                    liElement.appendChild(imgEl);
                    ulElement.appendChild(liElement);
                }
                div.appendChild(ulElement);
            });

        })
        .catch((error) => {
            console.error("Error:", error);
            document.getElementById('errormessage').innerHTML = error;
        });
};


document.addEventListener('DOMContentLoaded', function () { initpagescript("") });

function showOverlay() {
    document.getElementById('overlay').style.display = 'flex';
}

function hideOverlay() {
    document.getElementById('overlay').style.display = 'none';
}

function isNullOrEmpty(str) {
    return str === null || str === undefined || String(str).trim() === '';
}